///////////////////////////////////////////////////////////////////////////
//
//  ct_graphics.h   graphics routines to emulate CoreTools 
//                  Uses GLOBALs from ct_windows.h
//
////////////////////////////////////////////////////////////////////////////

#include "ct_windows.h"
#include "ct_graphics.h"

// All small functions SHOULD be inline. Duh.

// make a pen and a brush of this color and select into DC

// BAD: currentBrush and currentPen get selected too many times
//      when they are already in the DC. Fix this somehow.
//      IT'S OK, it seems.

void SetForeColor( int r, int g, int b ){
    SetForeColor( RGB( r, g, b ) );      // use the function below
}

void SetForeColor( COLORREF rgbc ){  

   HDC dc1 = GetDC( hGraphicWindowHandle );

   HPEN    hPrevPen    = hCurrentPen;
   HBRUSH  hPrevBrush  = hCurrentBrush;

   hCurrentPen = CreatePen( PS_SOLID,                // solid, not dashed 
                            0,                       // always 1 pixel wide
                            rgbc );
      
   hCurrentBrush = CreateSolidBrush( rgbc );  
   
   // select new stuff into DC, delete old stuff    
   HPEN   hOldPen   = (HPEN)   SelectObject( dc1, hCurrentPen );
   HBRUSH hOldBrush = (HBRUSH) SelectObject( dc1, hCurrentBrush );
   
   // delete the previous brush and the old pen -- unselect from DC first
   DeleteObject( hPrevPen );
   DeleteObject( hPrevBrush );

   // MSDN says, 
   //    An application should always replace a new object with the original, 
   //    default object after it has finished drawing with the new object. 
   SelectObject( dc1, hOldPen );
   SelectObject( dc1, hOldBrush ); 

   ReleaseDC( hGraphicWindowHandle , dc1 );
}

void PaintCircle( const Point& P, int r ){
    PaintCircle( P.h, P.v, r );
}

void PaintCircle( int x, int y, int r ){
    
   HDC dc1 = GetDC( hGraphicWindowHandle );
   
   HPEN   hPrevPen   =  (HPEN)   SelectObject( dc1, hCurrentPen );
   HBRUSH hPrevBrush =  (HBRUSH) SelectObject( dc1, hCurrentBrush );

   Ellipse( dc1, x-r, y-r, r + x, r+y );

   SelectObject( dc1, hPrevPen );
   SelectObject( dc1, hPrevBrush ); 
   ReleaseDC( hGraphicWindowHandle , dc1 );

}

void FrameCircle( const Point& P, int r ){
    FrameCircle( P.h, P.v, r );
}

void FrameCircle( int x, int y, int r ){
    
   HDC dc1 = GetDC( hGraphicWindowHandle );

   HPEN   hPrevPen   =  (HPEN)   SelectObject( dc1, hCurrentPen );
   HBRUSH hPrevBrush =  (HBRUSH) SelectObject( dc1, GetStockObject(NULL_BRUSH));
   
   Ellipse( dc1, x-r, y-r, r + x, r+y );

   SelectObject( dc1, hPrevPen );
   SelectObject( dc1, hPrevBrush );  // give back the previous values (MSDN)
   ReleaseDC( hGraphicWindowHandle , dc1 );

}

void PaintRect( const Point& UL, const Point& LR ){
    PaintRect( UL.h, UL.v, LR.h, LR.v );
}

void PaintRect( const Rect& R ){
    PaintRect( R.left, R.top, R.right, R.bottom );
}

void PaintRect( int x1, int y1, int x2, int y2 ){
    
   HDC dc1 = GetDC( hGraphicWindowHandle );
   
   HPEN   hPrevPen   =  (HPEN)   SelectObject( dc1, hCurrentPen );
   HBRUSH hPrevBrush =  (HBRUSH) SelectObject( dc1, hCurrentBrush );

   Rectangle( dc1, x1, y1, x2, y2 );

   SelectObject( dc1, hPrevPen );
   SelectObject( dc1, hPrevBrush ); 
   ReleaseDC( hGraphicWindowHandle , dc1 );

}

void FrameRect( const Point& UL, const Point& LR ){
    PaintRect( UL.h, UL.v, LR.h, LR.v );
}

void FrameRect( const Rect& R ){
    PaintRect( R.left, R.top, R.right, R.bottom );
}

void FrameRect( int x1, int y1, int x2, int y2 ){
    
   HDC dc1 = GetDC( hGraphicWindowHandle );
   
   HPEN   hPrevPen   =  (HPEN)   SelectObject( dc1, hCurrentPen );
   HBRUSH hPrevBrush =  (HBRUSH) SelectObject( dc1, GetStockObject(NULL_BRUSH));

   Rectangle( dc1, x1, y1, x2, y2 );

   SelectObject( dc1, hPrevPen );
   SelectObject( dc1, hPrevBrush ); 
   ReleaseDC( hGraphicWindowHandle , dc1 );

}

void ClearDrawing(){

   HDC dc1 = GetDC( hGraphicWindowHandle );

   HBRUSH hPrevBrush =  (HBRUSH) SelectObject( dc1, GetStockObject(WHITE_BRUSH) );

   Rectangle( dc1, 0, 0, GRAPHICS_WINDOW_WIDTH , GRAPHICS_WINDOW_HEIGHT ); 

   SelectObject( dc1, hPrevBrush ); 
   ReleaseDC( hGraphicWindowHandle , dc1 );

}

void MoveTo( const Point& P ){
    MoveTo( P.h, P.v );
}

void MoveTo( int x, int y ){
    HDC dc1 = GetDC( hGraphicWindowHandle );
        
    MoveToEx( dc1, x, y, NULL );
	
	// remember the pen position as our current pen point
	currentPenPointX = x;
    currentPenPointY = y;
    
	ReleaseDC( hGraphicWindowHandle , dc1 );
}

void LineTo( const Point& P ){
    LineTo( P.h, P.v );
}

void LineTo( int x, int y ){
    HDC dc1 = GetDC( hGraphicWindowHandle );
    
	HPEN hPrevPen = (HPEN) SelectObject( dc1, hCurrentPen );
    
	// move the pen to the current position, in case
	// the actual windows current point was changed
	MoveToEx( dc1, currentPenPointX, currentPenPointY, NULL );  	
	
	LineTo( dc1, x, y );

	// remember the pen position as our current point
	currentPenPointX = x;
    currentPenPointY = y;

    SelectObject( dc1, hPrevPen );   // give back the prev pen
    ReleaseDC( hGraphicWindowHandle , dc1 );
}

void DrawLine( const Point& P1, const Point& P2 ){
    DrawLine( P1.h, P1.v, P2.h, P2.v );
}

void DrawLine( int x1, int y1, int x2, int y2 ){
    HDC dc1 = GetDC( hGraphicWindowHandle );
    
	HPEN hPrevPen = (HPEN) SelectObject( dc1, hCurrentPen );
    
	MoveToEx( dc1, x1, y1, NULL );  	
	
	LineTo( dc1, x2, y2 );

    SelectObject( dc1, hPrevPen );   // give back the prev pen
    ReleaseDC( hGraphicWindowHandle , dc1 );
}




// Move and Line move and draw a line segment RELATIVE to the current point

void Move( int x, int y ){
    HDC dc1 = GetDC( hGraphicWindowHandle );
        
	MoveToEx( dc1, currentPenPointX + x, currentPenPointY + y, NULL );  	
		
	// remember the pen position as our current pen point
	currentPenPointX += x;
    currentPenPointY += y;
    
	ReleaseDC( hGraphicWindowHandle , dc1 );
}

void Line( int x, int y ){
    HDC dc1 = GetDC( hGraphicWindowHandle );
    
	HPEN hPrevPen = (HPEN) SelectObject( dc1, hCurrentPen );
    
	// move the pen to the current position, in case
	// the actual windows current point was changed
	MoveToEx( dc1, currentPenPointX, currentPenPointY, NULL );  	
	
	LineTo( dc1, currentPenPointX + x, currentPenPointY + y );

	// remember the pen position as our current point
	currentPenPointX += x;
    currentPenPointY += y;

    SelectObject( dc1, hPrevPen );   // give back the prev pen
    ReleaseDC( hGraphicWindowHandle , dc1 );
}

void SetColorPixel( int x, int y, short r, short g, short b ){
    HDC dc1 = GetDC( hGraphicWindowHandle );

    SetPixel( dc1, x, y, RGB( r, g, b ));

    ReleaseDC( hGraphicWindowHandle , dc1 );
}

void SetColorPixel( int x, int y, COLORREF rgbc ){
    HDC dc1 = GetDC( hGraphicWindowHandle );

    SetPixel( dc1, x, y, rgbc );

    ReleaseDC( hGraphicWindowHandle , dc1 );
}

void GetColorPixel( int x, int y, int& r, int& g, int& b ){
    HDC dc1 = GetDC( hGraphicWindowHandle );

    COLORREF rgbcolor;   // a 32-bit value, DWORD -- see RGB above 
    rgbcolor = GetPixel( dc1, x, y );

    r = rgbcolor & 0xFF;
    g = (rgbcolor >> 8 ) & 0xFF;
    b = (rgbcolor >> 16) & 0xFF;

    ReleaseDC( hGraphicWindowHandle , dc1 );
}

// Is this point inside this Rect?
// Mirrors the functionality of win32 
// BOOL PtInRect( CONST RECT *lprc,  // pointer to structure with rectangle
//                POINT pt           // structure with point
// );

// The point is considered to lie inside the rectangle if it lies in the 
// interior or the rectangle, or on ANY of its sides, corners included.
// This is different from PtInRect, which considers the points on the bottom
// and on the right sides of the rectangle to be OUTSIDE. 

bool PointInRect(const Point& P, const Rect& R) {
   return ( R.left <= P.h && P.h <= R.right && R.top <= P.v && P.v <= R.bottom ); 
}

// Point and Rect manipulations

// Create or modify points

void SetPoint(Point& P, int x, int y) {
	P.h = x;
	P.v = y;
}

// P.x and P.y are public, this function is for compatibility

void GetPoint(const Point& P, int& x, int& y) {
	x = P.h;
	y = P.v;
}

// You can use Point( x, y ) instead.

Point MakePoint(int x, int y) {
	Point P;
	P.h = x;
	P.v = y;
	return P;
}


// Create or modify rectangles

void SetRect(Rect& R, int x1, int y1, int x2, int y2) {
	R.left		= x1;
	R.top		= y1;
	R.right		= x2;
	R.bottom	= y2;
}

void GetRect(const Rect& R, int& x1, int& y1, int& x2, int& y2) {
	x1 = R.left;
	y1 = R.top;
	x2 = R.right;
	y2 = R.bottom;
}

// You can use Rect( x1, y1, x2, y2 ) instead.

Rect MakeRect(int x1, int y1, int x2, int y2) {
	Rect R;
	R.left		= x1;
	R.top		= y1;
	R.right		= x2;
	R.bottom	= y2;
	return R;	
}

// You can use Rect( P1, P2 ) instead.

Rect MakeRect(Point P1, Point P2) {
	return MakeRect(P1.h, P1.v, P2.h, P2.v);
}

